/*
 *	%Z% %M% %I% %D% %Q%
 *
 *	Copyright (C) The Santa Cruz Operation, 1985, 1986.
 *	This Module contains Proprietary Information of
 *	The Santa Cruz Operation, Microsoft Corporation
 *	and AT&T, and should be treated as Confidential.
 *
 */

/*
 * header file for sco standard bad track scheme
 */

/*
 * minor device number macros
 */
#define dmask(x)           ((1 << (x)) - 1)
#define	dphys(ptr,x)	   (((unsigned)(x) >> ( 8 - (ptr)->physbits)) & \
				  dmask((ptr)->physbits))
#define	dlog(ptr,x)	   (((unsigned)(x) >> (ptr)->partbits) & \
                                  dmask((ptr)->logbits))
#define	dpart(ptr,x)	    ((unsigned)(x) & \
                                  dmask((ptr)->partbits)) 
#define	dmkdev(ptr,d,l,p)  (((unsigned)(d) << (8 - (ptr)->physbits)) | \
                            ((unsigned)(l) << (ptr)->partbits) | \
                            ((unsigned)(p)))

/*
 * ioctl interface
 */
#define	DIO		('D'<<8)
#define	DIOGETP		(DIO|1)		/* get disk parameters */
#define	DIOSETP		(DIO|2)		/* set disk parameters */
#define	DIORPART	(DIO|3)		/* read partition table */
#define	DIOWPART	(DIO|4)		/* write partition table */
#define	DIORBTRK	(DIO|5)		/* read bad track table */
#define	DIOWBTRK	(DIO|6)		/* write bad track table */
#define DIOBITP         (DIO|7)         /* get bit parameters */
#define DIORVDT         (DIO|8)         /* read virt. drive table */
#define DIOWVDT         (DIO|9)         /* write virt. drive table */

/*
 * table sizes
 */
#define MAXPARTS   16                       /* max partitions possible*/
#define MAXALTS    126                      /* max alternate tracks possible */
#define MAXVDRIVES 16                       /* max virtual drives possible */

#define NPARTS(ptr) (1 << ((ptr)->partbits))/* this dev's # of partitions   */
#define NVIRTS(ptr) (1 << ((ptr)->logbits ))/* this dev's # of virt. drives */
#define NPHYS(ptr)  (1 << ((ptr)->physbits))/* this dev's # of phys drives  */

/*
 * magic numbers
 */
#define	PAMAGIC	0x1234	/* magic number for partition table */
#define	BAMAGIC	0x4321	/* magic number for bad track table */
#define	VAMAGIC	0x1342	/* magic number for virtual drive table */

/*
 * table format for DIOGETP, DIOSETP
 */
struct dparam {
	unsigned short d_cylin;		/* cylinders per drive */
	unsigned short d_heads;		/* heads per drive */
	unsigned short d_sectors;	/* sectors per cylinder */
	unsigned short d_bytes;		/* bytes per sector */
	unsigned short d_reserved;      /* sectors reserved for system use */
					/* at beginning of disk */
};

/*
 * table format for DIORPART, DIOWPART
 * partitions of zero size are considered non-existant
 */
struct partable {
	short p_magic;		/* magic number validity indicator */
	struct parts {
		long p_off;	/* offset of partition */
		long p_siz;	/* size of partition */
	} p[MAXPARTS];
};

/*
 * table format for DIORBTRK, DIOWBTRK
 * alternate list stored in sorted order by b_trk terminated
 * by a -1 in b_trk
 */
 struct badtab {
	unsigned short b_magic;
	unsigned short b_maxbad; 
	struct badtrks {
		unsigned short b_trk;
		unsigned short b_alt;
	} b[MAXALTS];
};

/* table format for DIOBITP
 * Get makeup of minor device number, which is how many bits of the
 * number are the partition number, how many are the virtual drive,
 * and how many are the physical drive
 */
struct dbitparams {
	short partbits;
	short logbits;
	short physbits;
};


/* table format for DIORVDT, DIOWVDT
 * Get/set the contents of the virtual drive table
 * which lists what operating system can be found
 * in each virtual drive, and the sizes of each.
 */

#define NOTACTIVE 0x00
#define ACTIVE    0x80

#define NO_OS        0
#define DOS          1
#define DOSL         4		/* different number for "large" DOS */
#define XENIX        2
#define CCPM         0xDB
#define AVIRT	     5		/* active virtual drive (bootable) */
#define DOSVIRT	     6		/* DOS virtual drive */

#define GDTRKS       4  /* first GDTRKS tracks of virt. drive must be flawless*/

struct vdtab {
	int  v_magic;
	int  v_nvds;
	struct vdisk {
		int v_active;
		int v_os;
		long v_off;
		long v_size;
		long  v_reserved;       /* sectors reserved for system use */
	} v_vdisk[MAXVDRIVES];
};

