/*
 *	%Z% %M% %I% %D% %Q%
 *
 *	Copyright (C) The Santa Cruz Operation, 1985, 1986.
 *	This Module contains Proprietary Information of
 *	The Santa Cruz Operation, Microsoft Corporation
 *	and AT&T, and should be treated as Confidential.
 *
 */

/*
 * THIS FILE CONTAINS CODE WHICH IS SPECIFIC TO THE
 * INTEL 80286 OR 8086 CPU AND MAY REQUIRE MODIFICATION
 * WHEN ADAPTING XENIX TO NEW HARDWARE.
 */

/*
 * Adjustable parameters
 */
#ifdef M_I86
#define	HZ	20		/* Ticks/second of the clock */
#define	SSIZE	1024		/* initial stack size (bytes) */
#endif

#ifdef M_I286
#define	HZ	50		/* Ticks/second of the clock */
#define	SSIZE	4096		/* initial stack size (bytes) */
#endif

#define	SINCR	1024		/* increment of stack (bytes) (not used)*/
#define	NOFILE	20		/* max open files per process */
#define	CDLIMIT	(1L<<13)	/* default max write address */
#define	CANBSIZ	256		/* max size of typewriter line */
#define	MSGBUFS	128		/* Characters saved from error messages */
#define	NCARGS	5120		/* # characters in exec arglist */
#define	MAXTTYS	16		/* Max # open ttys */
#define	NIOSTAT	50		/* max number of bufs to keep stats for */
#define	MAXPID	30000		/* max process id */
#define	MAXUID	60000		/* max user id */
#define	MAXLINK	1000		/* max links */

#define MINBUF	4		/* minimum # of buffers */
#define WUBSLOP 3072		/* 3k recoverable from wub */
#define NWUBBUF WUBSLOP/BSIZE	/* get 3 buffers from wub */ 
#define	NIDTBUF	(IDTLIM+1)/BSIZE /* get some buffers from idt */
#define KSSIZE	1024		/* kernel stack size */
#define OFFUSRPG 0		/* address of U page (for fpsup.s) */
#define BSSIZE   120		/* size of bootstr ( bs.c ) */

/*
 * priorities
 * probably should not be
 * altered too much
 */

#define	PSWP	0
#define	PINOD	10
#define	PRIBIO	20
#define	PZERO	25
#define	NZERO	20
#define PCATCH  0400
#define PMASK	0177	
#define	PPIPE	26
#define	PWAIT	30
#define	PSLEP	40
#define	PUSER	50
#define	PIDLE	127

/*
 * No more than 32 signals (1-32) because they are
 * stored in bits in a long.
 */

#include "signal.h"

#ifdef M_I86
#define	FSIG_DFL	((laddr_t ) 0)
#define	FSIG_IGN	((laddr_t) 1)
#endif

#ifdef M_I286
#define	FSIG_DFL	((faddr_t ) 0)
#define	FSIG_IGN	((faddr_t) 1)
#endif

/*
 * fundamental constants of the implementation--
 * cannot be changed easily
 */

#define	NBPW	sizeof(int)	/* number of bytes in an integer */
	/*
	 *      *** WARNING ***
	 * Some io devices may have a problem 
	 * if BSIZE is greater than MMPGSZ	
	 * If BSIZE changes, so must NSBFILL, BSHIFT,
	 * NSHIFT, LINOPB
         */
#define	BSIZE	1024		/* size of secondary block (bytes) */
#define	BSHIFT	10		/* LOG2(BSIZE) */
#define	BMASK	(BSIZE - 1)	/* BSIZE-1 */
#define	BSLOP	0		/* must be 0 for current copyio implemtation*/
#define	NINDIR	(BSIZE/sizeof(daddr_t))
#define	NSHIFT	8		/* LOG2(NINDIR) */
#define	NMASK	(NINDIR - 1)	/* NINDIR-1 */
#define	INOPB	(BSIZE/sizeof(struct dinode))	/* # inodes per block */
#define	LINOPB	4		/* LOG2(INOPB) */
#ifndef NULL
#define	NULL	0
#endif
#define	NODEV	(dev_t)(-1)
#define	ROOTINO	((ino_t)2)	/* i number of all roots */
#define	SUPERB	((daddr_t)1)	/* block number of the super block */
#define	DIRSIZ	14		/* max characters per directory */
#define	NICINOD	100		/* number of superblock inodes */
#define	NICFREE	100		/* number of superblock free blocks */
#define	NSBFILL	371		/* aligns s_magic & s_type at end of SB blk */
#define NSEG	2		/* max seg/user (see user.h) 8086 only */

/*
 * MMU parameters.
 */

#ifdef M_I86
#define	MMPGSZ	512		/* bytes/page in the MMU */
#define	PAGESIZE 16		/* size of memory allocation unit */
#define	PAGEMASK (PAGESIZE - 1)	/* PAGESIZE - 1 */
#define	LPGSIZE	4		/* log 2 of pagesize */
#define LMMPGSZ LPGSIZE
#define MMPGMASK PAGEMASK
#define NPAGEPS		0x1000		/* 64k/MMPGSZ = number of pages/seg*/
#define MAXPAGES  	0xA000
#define MAXPAGE  	MAXPAGES
#define UPGSIZE	sizeof(struct user)
#define USIZE	btoms(UPGSIZE)
#define	STACKSTART(ss)	(USRSTACK - mstob(ss))
#endif

#ifdef M_I286
#define	MMPGSZ	512		/* bytes/page in the MMU */
#define	LMMPGSZ	9		/* log2(MMPGSZ) */
#define NPAGEPS	128		/* 64k/MMPGSZ = number of pages per segment */
#define MAXPAGES  32768L
#define USIZE 	(u.u_procp->p_usize)
#endif

#ifndef M_I86
#ifndef M_I286
ERROR ****)))))
#endif
#endif

/*
 * Some macros for units conversion
 */

/*
 * mem size to disk blocks and disk blocks to mem size.
 *	Assumes BSIZE and MMPGSZ are both powers of 2.
 */
#ifdef M_I286
#define 	mstod(x)	( ((x) + ((BSIZE/MMPGSZ)-1)) / (BSIZE/MMPGSZ))
#define 	dtoms(x)	((x)*(BSIZE/MMPGSZ))
#endif

#ifdef M_I86
#define	mstod(x)  ((((x) + BSIZE/PAGESIZE - 1) >> (BSHIFT-LPGSIZE)) & 0x3FF)
#define	dtoms(x)  ((x) << (BSHIFT-LPGSIZE))
#endif

/* inumber to disk address */
#define	itod(x)	(daddr_t)(((unsigned)(x)+(INOPB+INOPB-1))>>LINOPB)

/* inumber to disk offset */
#define	itoo(x)	(int)(((x)+(INOPB+INOPB-1))&(INOPB-1))

#ifdef M_I86
/* bytes to mem size - this macro rounds up to a disk block boundary */
/* process memory is allocated only in BSIZE chunks */
#define btoms(x)  (msize_t)((x) ? (((((x)-1) | BMASK) >> LMMPGSZ) + 1) : 0)

/* bytes to mem size (truncated)*/
#define	btomst(x)	((unsigned)(x) >> LMMPGSZ)

/* bytes to page number */
#define	btopn(x)	(((unsigned)(x)) >> LMMPGSZ)

/* bytes to pages */
#define btop(x)	(((((unsigned)(x) - 1) | MMPGMASK) + 1) >> LMMPGSZ)

/* pages to bytes */
#define ptob(x)	((x) << LMMPGSZ)
#endif

#ifdef M_I286
/* bytes to mem size */
#define	btoms(x)      ((unsigned) (((long)(x) + (MMPGSZ-1)) >> LMMPGSZ))
#endif

/* mem size to bytes */
#define	mstob(x)	((x)<<LMMPGSZ)

/* address (long (32 bit)) to page number (int)*/
#define	atopn(x)	((mloc_t)((unsigned)((x) >> LMMPGSZ)))

#ifdef M_I286
/* address (long (32 bit)) to page count (int)*/
#define	atop(x)	((mloc_t)((unsigned)((x)+(MMPGSZ-1) >> LMMPGSZ)))

/* address (long (32 bit)) to page count (int)*/
#define	atoml(x)	((int)(((paddr_t)(x)+(MMPGSZ-1))>>LMMPGSZ))

/* address (long (32 bit)) to offset (int) get bits LMMPGSZ-1 - 0 */
#define	atoo(x)		((int)((x)&(MMPGSZ-1)))
#endif

/* address (long (32 bit)) to page number (int)*/
#define	atomlt(x)	((int)(((paddr_t)(x))>>LMMPGSZ))


#ifdef M_WORDSWAP /* LITTLE ENDIAN ( low word, high word in mem ) */

/* far pointer (faddr_t) to segment (int) (get high 16 bits of faddr_t) */
#define	ftoseg(x)		((unsigned)((long)(x) >> 16))
/* far pointer (faddr_t) to offset (int) */
#define	ftooff(x)		((unsigned)((long)(x)))
/* seg off to far pointer (faddr_t) */
#define sotofar(seg, off) \
	(((char far *) (((long)(unsigned) (seg)) << 16)) + (unsigned) (off))
/* mem loc to segaddr - returns segment = ml, offset = 0 */
#define mltof(x)	((long) (x) << 16 )
/* physical address (long 20 bit) to segaddr - offset/seg (in a long)*/
#define ptof(x) 	(((x) & 0x0000000f) | ((x << 12) & 0xffff0000))
/* real mode far pointer to physical address */
#define ftop(f) \
	((paddr_t) ((((long) (f) >> 12) & 0x000ffff0L) + (ushort) (f)))
/* physical address to real mode far pointer */
#define ptofar(p) \
	(((faddr_t) (((long) (p) << 12) & 0xffff0000L)) + ptooff(p))

#else /* BIG ENDIAN ( high word, low word in mem ) */

#define	ftoseg(x)		((unsigned)((long)(x)))
#define	ftooff(x)		((unsigned)((long)(x) >> 16))
#define sotofar(seg, off) \
	((char far *) ((long)(seg) + (unsigned) (off)))
#define mltof(x)	((long) (x))
#define ptof(x) 	(((x << 16) & 0xf0000) | ((x >> 4) & 0xffffL))
/* real mode far pointer to physical address */
#define ftop(f) \
((paddr_t) ((((long) (f) >> 16) & 0xffffL) + ((long)(f) << 4) & 0xffff0L))
/* physical address to real mode far pointer */
#define ptofar(p) \
	(((faddr_t) (((long) (p) >> 4) & 0xffffL)) + (((p) << 16) & 0xf0000L))
#endif

/* when sotofar() is used to set up an (faddr_t) parameter to a function call */
#define FADDR(seg, off)		off, seg

/* real mode seg, off to physical address */
#define sotop(seg, off) \
	((((paddr_t) (ushort) (seg)) << 4) + (ushort) (off))

/* physical address to real mode seg, off */
#define ptoseg(p)	((ushort) ((p) >> 4))
#define ptooff(p)	((ushort) (p) & 0x000f)


#ifdef M_I286
/* selector number to table index */
#define stoi(sel)		((sel)/SELSZ)
#endif

/* Memory location to logical address */
#define mltoa(x) ((paddr_t) (x) << LMMPGSZ)

/* High word of a long */
#define HIGHWORD(x)	((int)((long)x >> 16))

/* low word of a long */
#define LOWWORD(x)	((int)x)

#ifdef M_I86
/* physical to kernel logical (only the low 16 bits count) */
#define ptok(x) ((caddr_t)x)

/* kernel logical to physical */
#define ktop(x) (paddr_t)((unsigned)x)
#endif

#ifdef M_I286
/* physical to kernel logical (only the low 16 bits count) */
#define ptok(x) ((caddr_t)((unsigned) x - (unsigned) kmemstart))

/* kernel logical to physical */
#define ktop(x) ((paddr_t)(unsigned) x + kmemstart)
#endif

/* true if bp is not mapped out */
#define issabuf(bp)	(bp->b_paddr<=(paddr_t)ktop((unsigned)sabuf[v.v_sabuf]))

#define lobyte(X)	(((unsigned char *)&X)[0])
#define hibyte(X)	(((unsigned char *)&X)[1])

#ifdef M_KERNEL
#include	"types.h"
#endif

extern paddr_t kmemstart;	/* set by init code, passed in from /boot */


/*
 * Machine-dependent bits and macros
 */

#ifdef M_I86
#define copypsize(rpp,rip)	rpp->p_size = rip->p_size;
#endif

#ifdef M_I286
#define copypsize(rpp,rip)	rpp->p_usize = rip->p_usize; \
				rpp->p_lsize = rip->p_lsize
#endif

#ifdef M_I86
#define	URSBC		14		/* User stk Reg Save area Byte Cnt */
#define SHWDSZ          (sizeof(int)) /* space on user stack for shell word */
#endif

#ifdef M_I286
#define	URSBC		0		/* User stk Reg Save area Byte Cnt */
#define SHWDSZ          (sizeof(int)+6) /* space on user stack for shell word */
#define	USRSTACK	u.u_stkbot
#endif

#define CLKONLY(ps)     (((ps)&PS_PRIMASK)==0)
#define USERMODE(ps)	((ps)&PS_USER)


#define PTRSIZE(f)	((f)? sizeof(char far *) : sizeof(caddr_t))

/*
 * 286/8086 simulated "state" definitions.
 */

#define	PS_USER		0x01		/* "user" mode */
#define	PS_SYS		0xFE		/* for mch.a86, turns off user mode */
#define	PS_PRIMASK	0xFF00		/* previous priority */

/* Conditional compilation */
#define	PAGEMMU				/* mmu is divided in pages */
#define	SHORTINT			/* sizeof(int)==sizeof(short) */
#define BUFMAPOUT      		        /* buffers are mapped out */
#define UNOTMAPPED			/* upage copied in on context switch */

#ifdef M_I86
#define LARGECODE
#endif

#ifdef M_I286
#define FIXEDSTACK 	                /* stack cannot grow */
#endif

