
/*
 *	%Z% %M% %I% %D% %Q%
 *
 *	Copyright (C) The Santa Cruz Operation, 1985, 1986.
 *	This Module contains Proprietary Information of
 *	The Santa Cruz Operation, Microsoft Corporation
 *	and AT&T, and should be treated as Confidential.
 *
 */

/*
 * THIS FILE CONTAINS CODE WHICH IS SPECIFIC TO THE
 * INTEL 80286 CPU AND MAY REQUIRE MODIFICATION
 * WHEN ADAPTING XENIX TO NEW HARDWARE.
 */


/*
 * One structure allocated per active process. It contains all data needed
 * about the process while the process may be swapped out.
 * Other per process data (user.h) is swapped with the process.
 */

struct	proc {
	char	p_stat;
	char	p_flag;
	char	p_pri;		/* priority, negative is high */
	char	p_time;		/* resident time for scheduling */
	char	p_cpu;		/* cpu usage for scheduling */
	char	p_nice;		/* nice for cpu usage */
	long	p_sig;		/* signals pending to this process */
	ushort	p_uid;		/* real user id */
	ushort	p_suid;		/* saved-set/effective uid */
	ushort	p_sgid;		/* saved-set/effective gid */
	short	p_pgrp;		/* name of process group leader */
	short	p_pid;		/* unique process id */
	short	p_ppid;		/* process id of parent */
	union {
		unsigned short p_daddr;	/* block number of u on swap */
		mloc_t	p_caddr;	/* location of p_usize region in core */
	} p_addr;
	unsigned short p_fcsel;  /* first code selector: 286 only */
	unsigned short p_fdsel;  /* first data selector: 286 only */
	unsigned short p_ldsel;  /* last data selector: 286 only */
	msize_t	p_usize;	 /* size of u + ldts + ldtinfo seg: 286 only */
	unsigned short p_lsize;  /* size in bytes of the ldt: 286 only */
	caddr_t p_wchan;	 /* event process is awaiting */
	struct text *p_textp;	 /* pointer to text structure */
	struct proc *p_link;	 /* linked list of running processes */
	int	p_clktim;	 /* time to alarm clock signal */
	struct sd *p_sdp;	 /* pointer to list of shared data segments */
	msize_t p_size;		/* size of swappable image (clicks) 8086 only*/
	char    p_cflags;       /*  contiguous memory flags 8086 only*/
	char 	p_fill;		 /* padding */
};

extern struct proc proc[];	/* the proc table itself */


/* stat codes */
#define	SSLEEP	1		/* awaiting an event */
#define	SWAIT	2		/* (abandoned state) */
#define	SRUN	3		/* running */
#define	SIDL	4		/* intermediate state in process creation */
#define	SZOMB	5		/* intermediate state in process termination */
#define	SSTOP	6		/* process being traced */

/* flag codes */
#define	SLOAD	01		/* in core */
#define	SSYS	02		/* scheduling process */
#define	SLOCK	04		/* process cannot be swapped */
#define	SSWAP	010		/* process is being swapped out */
#define	STRC	020		/* process is being traced */
#define	SWTED	040		/* another tracing flag */
#define	STEXT	0100		/* text pointer valid */
#define	SSPART	0200		/* process is partially swapped out */

/* contiguity flags */
#define	SCSTACK	01
#define	SCDATA	02
#define SCWANT	04
#define SCLDATA 010
#define SCLTEXT 020

/*
 * parallel proc structure
 * to replace part with times
 * to be passed to parent process
 * in ZOMBIE state.
 */
struct	xproc {
	char	xp_stat;
	char	xp_flag;
	char	xp_pri;		/* priority, negative is high */
	char	xp_time;	/* resident time for scheduling */
	char	xp_cpu;		/* cpu usage for scheduling */
	char	xp_nice;	/* nice for cpu usage */
	long	xp_sig;		/* signals pending to this process */
	ushort	xp_uid;		/* real user id */
	ushort	xp_suid;	/* saved-set/effective uid */
	ushort	xp_sgid;	/* saved-set/effective gid */
	short	xp_pgrp;	/* name of process group leader */
	short	xp_pid;		/* unique process id */
	short	xp_ppid;	/* process id of parent */
	short	xp_xstat;	/* Exit status for wait */
	time_t	xp_utime;	/* user time, this proc */
	time_t	xp_stime;	/* system time, this proc */
};
