
/*
 *	%Z% %M% %I% %D% %Q%
 *
 *	Copyright (C) The Santa Cruz Operation, 1985, 1986.
 *	This Module contains Proprietary Information of
 *	The Santa Cruz Operation, Microsoft Corporation
 *	and AT&T, and should be treated as Confidential.
 *
 */
/*
 * THIS FILE CONTAINS CODE WHICH IS SPECIFIC TO THE
 * INTEL 80286/8086 CPU AND MAY REQUIRE MODIFICATION
 * WHEN ADAPTING XENIX TO NEW HARDWARE.
 */

/*
** U area-
**	On the 286, we keep the kernel stack in the first part of
**	the u area, and the LDT for the process lives above it.
**	The kernel stack lives at the start of the u and grows down so that
**	if it overflows, we get a fault instead of trashing the u area.
**	Note that the LDT is not part of the u area proper,
**	it just tags along for the ride during swapping, etc.
**	When we save and resume we don't move the LDT, we just 
**	set the LDT register to point to wherever the LDT 
**	happens to be in memory.	
*/
/*
 * The user structure.
 * One allocated per process.
 * Contains all per process data that doesn't need to be referenced
 * while the process is swapped.
 * The user block is p_usize*click bytes long;
 * contains the system stack per user; is cross referenced
 * with the proc structure for the same process.
 */

#ifndef X_MAGIC
#include "a.out.h"
#endif
 
struct	user
{                               /* octal offset */
	char	u_kstack[KSSIZE];   /* kernel stack */
	label_t u_rsav;             /* save info when exchanging stacks */
	label_t u_qsav;             /* label variable for quits and interrupts*/
	label_t u_ssav;             /* label variable for swapping */
	char    u_segflg;           /* IO flag: 0:user D; 1:system; 2:user I */
	char    u_error;            /* return error code */
	ushort  u_uid;              /* effective user id */
	ushort  u_gid;              /* effective group id */
	ushort  u_ruid;             /* real user id */
	ushort  u_rgid;             /* real group id */
	struct proc *u_procp;       /* pointer to proc structure */
	int     *u_ap;              /* pointer to arglist */
	union {                     /* syscall return values */
		struct	{
			int     r_val1;  
			int     r_val2; 
		}r_reg;
		off_t	r_off;
		time_t	r_time;
		long	r_long;
		faddr_t	r_faddr;
		char far *r_far;
	} u_r;
	union {
		char far *bu_bf;
		char 	 *bu_bn;
	} u_baseu;			    /* base address for IO */
#ifdef M_I86
#  define 	u_base 	u_baseu.bu_bn
#else
#  define 	u_base 	u_baseu.bu_bf
#endif
	unsigned u_count;           /* bytes remaining for IO */
	off_t   u_offset;           /* offset in file for IO */
	short   u_fmode;            /* file mode for IO */
	short   u_errcnt;           /* syscall error count */
	struct inode *u_cdir;       /* pointer to inode of current directory */
	struct inode *u_rdir;       /* root directory of current process */
	union {
		char far *dpu_dpf;
		char	 *dpu_dpn;
	} u_dirpu;			    /* pathname pointer */
#ifdef M_I86
#  define 	u_dirp 		u_dirpu.dpu_dpn  
#else
#  define 	u_dirp 		u_dirpu.dpu_dpf
#endif
	struct direct u_dent;       /* current directory entry */
	struct inode *u_pdir;       /* inode of parent directory of dirp */
	struct file *u_ofile[NOFILE];/* pointers to file structures of open files */
	char    u_pofile[NOFILE];   /* per-process flags of open files */
	int     u_arg[16];          /* arguments to current system call */
	unsigned u_tsize;           /* text size (clicks) */
	unsigned u_dsize;           /* data size (clicks) */
	unsigned u_ssize;           /* stack size (clicks) */
	char far *u_signal[NSIG];   /* disposition of signals */
	time_t  u_utime;            /* this process user time */
	time_t  u_stime;            /* this process system time */
	time_t  u_cutime;           /* sum of childs' utimes */
	time_t  u_cstime;           /* sum of childs' stimes */
	int     *u_ar0;             /* address of users saved R0 */
	struct {                    /* profile arguments */
		char far *pr_base;
		unsigned pr_size;   /* buffer size */
		long     pr_off;    /* pc offset */
		unsigned pr_scale;  /* pc scaling */
		char far *pr_syspc;
	} u_prof;
	char    u_intflg;           /* catch intr from sys */
	char    u_sep;              /* flag for I and D separation */
	char	u_osvers;	    /* version of binary (SYS2, 3, 5, etc) */
	char	u_res1;		    /* padding */
	unsigned short	u_renv;	    /* runtime environment */ 
	struct tty *u_ttyp;         /* controlling tty pointer */
	dev_t   u_ttyd;             /* controlling tty dev */
	struct u_exdata {	    /* header information */
		unsigned short x_magic;
		unsigned short x_ext;	/* 8086 only */
		long	x_text;	    /* size of text segment(s) */
		long	x_data;	    /* size of initialized data segment(s) */
		long	x_bss;	    /* size of uninitialized data segment(s) */
		long	x_syms;		/* 8086 only */
		long	x_reloc;	/* 8086 only */
		long	x_entaddr;    /* entry address */
		char	x_cpu;		/* 8086 only */
		char	x_relsym;
		unsigned short x_renv; /* run-time environment */
		long	x_stack;    /* size of stack (if XE_FS set) */
		char	x_osvers;
		char 	x_res3;		/* padding */
		long	x_segsize;	/* segment table size */
		long	x_segpos;	/* segment table position */
	} u_exdata;
	unsigned u_hdrsiz;		/* 8086 only x.out hdr size */
	char    u_comm[DIRSIZ];     
	time_t  u_start;            
	time_t  u_ticks;           
	long    u_mem;            
	long    u_ior;           
	long    u_iow;          
	long    u_iosw;        
	long    u_ioch;       
	char    u_acflag;    
	char	u_nfsflgs;    	    /* reserved for future use */
	short   u_cmask;            /* mask for file creation */
	daddr_t u_limit;            /* maximum write address */
	int     u_t[32];            /* (see iupdat) */
	int     u_sdsaved;          /* we have saved the shared data */
/* machine dependent fields */
	int     u_fper;             /* FP error register */
	char    u_fpsaved;          /* FP regs saved for this proc */
	char    u_fpused;           /* FP used for this process    */
	struct {                    /* 80287 save status */
		int	fp_cntrl;	/* control word */
		int	fp_stat;	/* status word */
		int	fp_tag;		/* tag word */
		int	fp_ip[2];	/* instruction pointer */
		int	fp_op[2];	/* operand pointer */
		char    fp_regs[100];   /* temporary registers */
		int     fp_spare;       /*   (2 xtra for emulator)      */
	} u_fps;
	unsigned short  u_lxrw;     /* R/W access of text segments */ 
	union {
		char far *sbu_sbf;
		char 	 *sbu_sbn;
	} u_stkbotu;		    /* start or bottom of user stack */
#ifdef M_I86
#  define	u_stkbot	u_stkbotu.sbu_sbn
#else
#  define	u_stkbot	u_stkbotu.sbu_sbf
#endif
	union {
		char far *edu_edf;
		char 	 *edu_edn;
	} u_edatau;		   /* bottom of user data before brks */	
#ifdef M_I86
#  define	u_edata		u_edatau.edu_edn
#else
#  define	u_edata		u_edatau.edu_edf
#endif
	
	struct segtmp far *u_segoff;    /* ptr to segoff array used by exec */
	char	 u_hugex;	    /* allows exec of proc > swapper when set */
	char	 u_lock;	    /* plock info */
	/* following are 8086 only fields */
	short	u_usegs[NSEG];	    /* segments table for mem management*/
	caddr_t	u_topstk;		/* user data address of stack top*/
	caddr_t u_botstk;		/*   "   "     "      "   bottom*/
	unsigned u_flags;		/* misc flags */
	caddr_t u_seginfo;		/* seginfo table pointer */
	unsigned u_curbrk;		/* current break val: brkctl */
	char	u_end;
	char	u_res2;			/* padding */
};
extern struct user u;

#define	u_rval1	u_r.r_reg.r_val1
#define	u_rval2	u_r.r_reg.r_val2
#define	u_roff	u_r.r_off
#define	u_rtime	u_r.r_time
#define	u_rlong	u_r.r_long
#define	u_rfaddr u_r.r_faddr
#define u_rfar   u_r.r_far

/* u_flags */
#define UF_WSWAP	0x1	/* Words in longs swapped relative to kernel */

/* u_error codes */
#include "errno.h"

/* u_segflg values: Read/Write, User/Kernel, Ins/Data */
#define U_WRITE	0
#define U_READ	1

#define U_UD	0	/* user data */
#define U_KD	1	/* kernel data */
#define U_UI	2	/* user code */

#define	U_WUD	((U_UD << 1) | U_WRITE)		/* 0 */
#define	U_RUD	((U_UD << 1) | U_READ)		/* 1 */
#define	U_WKD	((U_KD << 1) | U_WRITE)		/* 2 */
#define	U_RKD	((U_KD << 1) | U_READ)		/* 3 */
#define	U_WUI	((U_UI << 1) | U_WRITE)		/* 4 */
#define	U_RUI	((U_UI << 1) | U_READ)		/* 5 */

#define	EXCLOSE	01

/* values for u_nfsflgs (all bits reserved for NFS) */
#define NFSNEGO   0x01		/* core negotiate wanted or not */
#define NFSNOCASE 0x80		/* caseless compare needed (core server proc)*/

/* values for u_hugex flag */
#define SMALL	  0x01		/* not currently bigger than swap area */
#define BIG	  0x02		/* proc is bigger than swap area*/
